;; Generated package description from auctex.el  -*- no-byte-compile: t -*-
(define-package "auctex" "14.0.0.0.20240208.115629" "Integrated environment for *TeX*" '((emacs "27.1")) :commit "c207068f5daa6221b2486e598fc75506ff1a2e93" :maintainer '(nil . "auctex-devel@gnu.org") :keywords '("tex" "latex" "texinfo" "context" "doctex" "preview-latex") :url "https://www.gnu.org/software/auctex/")
